@@ -3,7 +3,7 @@ Hessian Update Strategies
 -------------------------
 This module provides various generic Hessian approximation strategies that
 can be employed when the calculating the exact Hessian or an approximation
-is computationally too demandind.
+is computationally too demanding.
 """
 
 
@@ -21,24 +21,36 @@ class HessianApproximation:
         Create a Hessian update strategy instance
 
         :param hess_init:
-            Inital guess for the Hessian, if empty Identity matrix will be used
+            Initial guess for the Hessian, if empty Identity matrix will be
+            used
         """
+        self.hess_init = None
         if hess_init is not None:
-            if not isinstance(hess_init, np.ndarray):
-                raise ValueError('Cannot initialize with hess_init of type'
-                                 f'{type(hess_init)}, needs np.ndarray.')
+            self.set_init(hess_init)
+        self._hess = None
 
-            if not hess_init.ndim == 2:
-                raise ValueError('hess_init needs to be a matrix with'
-                                 f'hess_init.ndim == 2, was {hess_init.ndim}')
+    def set_init(self, hess_init: np.ndarray):
+        """
+        Create a Hessian update strategy instance
 
-            if not hess_init.shape[0] == hess_init.shape[1]:
-                raise ValueError('hess_init needs to be a square matrix!')
+        :param hess_init:
+            Initial guess for the Hessian, if empty Identity matrix will be
+            used
+        """
+        if not isinstance(hess_init, np.ndarray):
+            raise ValueError('Cannot initialize with hess_init of type'
+                             f'{type(hess_init)}, needs np.ndarray.')
 
-            hess_init = hess_init.copy()
+        if not hess_init.ndim == 2:
+            raise ValueError('hess_init needs to be a matrix with'
+                             f'hess_init.ndim == 2, was {hess_init.ndim}')
+
+        if not hess_init.shape[0] == hess_init.shape[1]:
+            raise ValueError('hess_init needs to be a square matrix!')
+
+        hess_init = hess_init.copy()
 
         self.hess_init: np.ndarray = hess_init
-        self._hess = None
 
     def init_mat(self, dim: int):
         """
@@ -52,7 +64,7 @@ class HessianApproximation:
         else:
             self._hess = self.hess_init.copy()
             if self._hess.shape[0] != dim:
-                raise ValueError('Inital approximation had inconsistent '
+                raise ValueError('Initial approximation had inconsistent '
                                  f'dimension, was {self._hess.shape[0]}, '
                                  f'but should be {dim}.')
 
@@ -118,16 +130,22 @@ class HybridUpdate(HessianApproximation):
     def __init__(self,
                  happ: HessianApproximation = None,
                  hess_init: Optional[np.ndarray] = None,
+                 init_with_hess: Optional[bool] = False,
                  switch_iteration: Optional[int] = None):
         """
         Create a Hybrid Hessian update strategy which is generated from the
-        start but only applied after a certain iteration
+        start but only applied after a certain iteration, while Hessian
+        computed by the objective function is used until then.
 
         :param happ:
             Hessian Update Strategy (default: BFGS)
 
         :param switch_iteration:
-            Iteration after which this approximation is used (default: 5*dim)
+            Iteration after which this approximation is used (default: 2*dim)
+
+        :param init_with_hess (default: False)
+            Whether the hybrid update strategy should be initialized
+            according to the user-provided objective function
 
         :param hess_init:
             Initial guess for the Hessian. (default: eye)
@@ -136,9 +154,16 @@ class HybridUpdate(HessianApproximation):
             happ = BFGS()
         self.hessian_update = happ
         self.switch_iteration = switch_iteration
+        self.init_with_hess = init_with_hess
+        if init_with_hess and hess_init is not None:
+            raise ValueError('init_with_hess cannot be set to true if '
+                             'hess_init is also provided.')
 
         super(HybridUpdate, self).__init__(hess_init)
 
+    def set_init(self, hess_init: np.ndarray):
+        self.hessian_update.set_init(hess_init)
+
     def init_mat(self, dim: int):
         if self.switch_iteration is None:
             self.switch_iteration = 2*dim
